﻿Type=Class
Version=4.7
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Private FM As Form
	Private mModule As Object
	Private mEventName As String
	Private CM As ContextMenu
	Type MenuItemType(MType As String,Text As String)
	
	Public Const MENUTYPE_SUBMENUTITLE As String = "title"
	Public Const MENUTYPE_SEPERATOR As String = "seperator"
	Public Const MENUTYPE_TEXT As String = "item" 
	Public Const MENUTYPE_CHECKBOX As String = "checkbox"
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Module As Object,EventName As String)
	mModule = Module
	mEventName = EventName
	
	'Create a placeholder pane so we can set the context menu next to it
	FM.Initialize("FM",1,1)
	
	'No control buttons on the form
	FM.SetFormStyle("UNDECORATED")
	Utils.AsJO(FM).GetFieldJO("stage").RunMethodJO("setAlwaysOnTop",Array(True))
	
	'Make the background transparent
	CSSUtils.SetBackgroundColor(FM.RootPane,fx.Colors.Transparent)
	
	CM.Initialize("CM")
End Sub

'Add text items to the context menu, as an array, Pass a '-' to add a seperator
'<code>FM.AddItems(Array As String("Test","-","Test1"))</code>
Sub AddItems(Items As List,Clear As Boolean)
	
	If Clear Then CM.MenuItems.Clear
	
	BuildContextMenu (Items)

End Sub

'Build TopLevel ContextMenu
Private Sub BuildContextMenu (Items As List)
	Dim MIT As MenuItemType
	
	'Populate the context menu
	For Each It As Object In Items
		If It Is List Then
			'Build a SubMenu
			CM.MenuItems.Add(BuildSubMenu(It))
		Else
			MIT = It
			Select MIT.MType
			Case MENUTYPE_SEPERATOR
				CM.MenuItems.Add(SeperatorMenuItem)
			Case MENUTYPE_CHECKBOX
				CM.MenuItems.Add(CheckMenuItem(MIT.Text))
			Case MENUTYPE_TEXT
				Dim MI As MenuItem
				MI.Initialize(MIT.Text,"MI")
				CM.MenuItems.Add(MI)
			End Select
		End If
	Next
	
End Sub

'Build and return a submenu
Sub BuildSubMenu(Items As List) As Menu
	'First entry in the list is the name for this sub menu
	Dim MIT As MenuItemType = Items.Get(0)
	If MIT.MType <> MENUTYPE_SUBMENUTITLE Then
		Log("Error *** : First entry of a sub menu must be a Title")
		'Should throw an exception
		Return Null
	End If
	Dim M As Menu
	M.Initialize(MIT.Text,"MI")
	
	'Process the real menu Items
	For i = 1 To Items.Size - 1
		Dim IT As Object = Items.Get(I)
		If IT Is List Then
			M.MenuItems.Add(BuildSubMenu(IT))
		Else
			'Delimeter
			MIT = IT
			Select MIT.MType
				Case MENUTYPE_SEPERATOR
					M.MenuItems.Add(SeperatorMenuItem)
				Case MENUTYPE_CHECKBOX
					M.MenuItems.Add(CheckMenuItem(MIT.Text))
				Case MENUTYPE_TEXT
					Dim MI As MenuItem
					MI.Initialize(MIT.Text,"MI")
					M.MenuItems.Add(MI)
			End Select
		End If
	Next
	Return M
End Sub

'Show the invisible dummy panel and the context menu to the right of it (will automatically reposition if it would be off the screen)
Sub Show(X As Int,Y As Int)
	FM.WindowLeft = X
	FM.WindowTop = Y
	FM.Show
	Utils.AsJO(CM).RunMethod("show",Array(FM.RootPane,"RIGHT",Utils.AsDouble(0),Utils.AsDouble(0)))
End Sub

'Hide the invisible panel and context menu
Sub Hide
	FM.Close
	Utils.AsJO(CM).RunMethod("hide",Null)
End Sub

'Menu item clicked, pass it back to the originating module
Sub MI_Action
	Hide
	Dim MI As MenuItem = Sender
	If SubExists(mModule,mEventName & "_Clicked") Then CallSub2(mModule,mEventName & "_Clicked",MI)
End Sub

'Checkbox Check Changed
Sub CBChanged_Event(MethodName As String,Args() As Object) As Object
	Dim MI As JavaObject = Sender
	If SubExists(mModule,mEventName & "_CheckChanged") Then CallSub3(mModule,mEventName & "_CheckChanged",MI.RunMethod("getText",Null),MI.RunMethod("isSelected",Null))
	Return True
End Sub

Public Sub NewMenuItem (MType As String,Text As String) As MenuItemType
	Dim MI As MenuItemType
	MI.Initialize
	MI.MType = MType
	MI.Text = Text
	Return MI
End Sub

'Create and return a SeperatorMenuItem Object
Private Sub SeperatorMenuItem As JavaObject
	Dim JO As JavaObject
	JO.InitializeNewInstance("javafx.scene.control.SeparatorMenuItem",Null)
	Return JO
End Sub

'Create and Return a CheckMenuItem and set a changelistener
Private Sub CheckMenuItem(Text As String) As JavaObject
	Dim JO As JavaObject
	JO.InitializeNewInstance("javafx.scene.control.CheckMenuItem",Array(Text))
	JO.RunMethod("setSelected",Array(True))
	Dim Event As Object = JO.CreateEventFromUI("javafx.beans.value.ChangeListener","CBChanged",False)
	JO.RunMethodJO("selectedProperty",Null).RunMethod("addListener",Array(Event))
	Return JO
End Sub